from django.conf import settings
from django.shortcuts import render as _render

from foledol.django.utils import paginate, get_param_from_get_or_request, new_context, get_action


class TableFilter:
    def __init__(self, key, label):
        self.key = key
        self.label = label


class TableButton:
    def __init__(self, label, action):
        self.label = label
        self.action = action


class TableButtonGroup:
    def __init__(self, label, items):
        self.label = label
        self.items = items


class TableButtonDivider:
    def __init__(self):
        None


class TableColumn:
    def __init__(self, key, name, type=None, value=None, method=None, link=None, sortable=False):
        self.key = key
        self.name = name
        self.type = type
        self.value = value
        self.method = method
        self.link = link
        self.sortable = sortable


class Table:
    def __init__(self, model, columns, rows=None, path=None, heading=None, create=None, update=None,
                 search=False, upload=False, filters=None, buttons=None, placeholder=None, sort='date_asc'):
        self.model = model
        self.columns = columns
        self.rows = rows
        self.count = rows.count if rows else 0
        self.path = path
        self.sort = sort
        self.title = None
        self.params = {}
        self.heading = heading
        self.create = create
        self.update = update
        self.search = search
        self.upload = upload
        self.filters = filters
        self.buttons = buttons
        self.template = 'table_view.html'
        self.placeholder = placeholder

    def sort(self, rows):
        pass

    def formatter(self, row):
        return ''


class TableView(Table):

    def all(self):
        return self.model.objects.all()

    def select(self, rows, search, order_by):
        return rows

    def handle(self, request, context, action, rows):
        return False

    def get_space(self):
        segment = self.path.split(':')
        return segment[1] if len(segment) > 1 else segment[0]

    def get_rows(self, request, context):
        space = self.get_space()
        search = get_param_from_get_or_request(request, context, space, 'search', '').strip()
        order_by = get_param_from_get_or_request(request, context, space, 'sort', self.sort)
        filter_key = get_param_from_get_or_request(request, context, space, 'filter_key', None)
        rows = self.all()
        if filter_key and self.filters and filter_key in self.filters:
            current_filter = self.filters[filter_key]
            rows = current_filter.filter(rows)
            context['filter_title'] = current_filter.title
        return self.select(rows, search, order_by)

    def render(self, request, context=new_context()):
        if context is None:
            context = new_context()
        rows = self.get_rows(request, context)
        space = self.get_space()
        self.rows = paginate(request, context, rows, space=space)
        self.count = rows.count()
        context['base'] = settings.DEFAULT_SPACE + '/base.html'
        context['title'] = self.title
        context['table'] = self

        action = get_action(request)
        context['action'] = ''
        result = self.handle(request, context, action, rows)
        if result:
            return result

        return _render(request, self.template, context)



from django.contrib.admin.views.decorators import staff_member_required
from django.contrib.auth.decorators import login_required
from foledol.django.tools.table import TableColumn, TableView

from ..models import Condition


class ConditionTables(TableView):
    def __init__(self):
        super().__init__(Condition, [
            TableColumn('label', "Libellé"),
            TableColumn('criteria', "Critère"),
            TableColumn('value', "Valeur")
        ], path='django:conditions', search=True)
        self.update = 'django:condition_update'
        self.create = 'django:condition_create'

    def select(self, conditions, search, order_by):
        return conditions.filter(label=search) if len(search) > 0 else conditions


@login_required
@staff_member_required
def condition_list(request):
    return ConditionTables().render(request)



from django.contrib.admin.views.decorators import staff_member_required
from django.contrib.auth.decorators import login_required
from foledol.django.tools.table import TableColumn, TableView

from ..models import Column


class ColumnTable(TableView):
    def __init__(self):
        super().__init__(Column, [
            TableColumn('label', "Libellé")
        ], path='django:columns', search=True)
        self.update = 'django:column_update'
        self.create = 'django:column_create'

    def select(self, columns, search, order_by):
        return columns.filter(label=search) if len(search) > 0 else columns


@login_required
@staff_member_required
def column_list(request):
    return ColumnTable().render(request)



from datetime import datetime

from django.contrib.auth.models import User

from .meta import get_user_or_superuser
from .models import Log, LogItem
from .utils import get_local_date


def log(ref, model, action, user=None, old='', new='', transaction=None):

    def too_big(value):
        return value and isinstance(value, str) and len(value) > 126

    _log = Log(user=get_user_or_superuser(user))
    _log.date = get_local_date()
    _log.ref = ref
    _log.model = model
    _log.action = action
    _log.transaction = transaction
    _log.save()

    if old and new:
        diff = {}
        for key in new.keys():
            if old[key] != new[key]:
                diff[key] = (old[key], new[key])
        for key in diff.keys():
            if not too_big(diff[key][0]) and not too_big(diff[key][1]):
                try:
                    _log_item = LogItem()
                    _log_item.log = _log
                    _log_item.key = key
                    _log_item.old = diff[key][0]
                    _log_item.new = diff[key][1]
                    _log_item.save()
                except Exception as ex:
                    print("Exception:" + str(ex))


def logs(ref, model):
    return Log.objects.all().filter(ref=ref).filter(model=model).order_by('-date')


